// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Text;

namespace Esmf
{
    public class ModelFields
    {

        public class Field
        {
            public string ComponentName { get; set; }
            public string FieldName { get; set; }
            public object Values { get; set; }
        }

        struct FieldKey
        {
            public FieldKey(string componentName, string fieldName)
            {
                _componentName = componentName;
                _fieldName = fieldName;
            }

            private string _componentName;
            private string _fieldName;

            public string ComponentName { get { return _componentName; } }
            public string FieldName { get { return _fieldName; } }
        }

        class FieldValue<T>
        {
            private T _value;
            public T Value
            {
                get { return _value; }
                set { _value = value; }
            }
        }

        private class DelegateToParameter1Dimensional<D1, T> : IParameter1Dimensional<D1, T>
        {
            Func<D1, T> _delegate;

            public DelegateToParameter1Dimensional(Func<D1, T> function)
            {
                _delegate = function;
            }

            T IParameter1Dimensional<D1, T>.this[D1 index]
            {
                get { return _delegate(index); }
            }

            public IEnumerable<Parameter1DimensionalMember<D1, T>> GetEnumerator()
            {
                throw new NotImplementedException();
            }
        }


        private class DelegateToParameter2Dimensional<D1,D2,T> : IParameter2Dimensional<D1,D2,T>
        {
            private ModelFields _parent;
            Func<D1, D2, T> _delegate;
            HashSet<D1> _d1Members = new HashSet<D1>();
            HashSet<D2> _d2Members = new HashSet<D2>();

            public DelegateToParameter2Dimensional(Func<D1, D2, T> function, ModelFields parent)
            {
                _delegate = function;
                _parent = parent;
            }

            T IParameter2Dimensional<D1, D2, T>.this[D1 index1, D2 index2]
            {
                get 
                {
                    _d1Members.Add(index1);
                    _d2Members.Add(index2);

                    return _delegate(index1, index2); 
                }
            }

            #region IParameter2Dimensional<D1,D2,T> Members


            public IEnumerable<Parameter2DimensionalMember<D1, D2, T>> GetEnumerator()
            {
                foreach (var d1 in _d1Members)
                {
                    foreach (var d2 in _d2Members)
                    {
                        yield return new Parameter2DimensionalMember<D1, D2, T>(d1, d2, _delegate(d1, d2), _parent.AttachedDimensions);
                    }
                }
            }

            #endregion
        }

        List<FieldKey> _variablesKeysAlreadyUsed = new List<FieldKey>();

        #region NonDimensionalVariables
        Dictionary<Type, object> _nonDimensionalVariables = new Dictionary<Type, object>();
        Dictionary<FieldKey, object> _nonDimensionalVariablesGetterDelegates = new Dictionary<FieldKey, object>();
        Dictionary<FieldKey, object> _nonDimensionalVariablesSetterDelegates = new Dictionary<FieldKey, object>();

        public void AddNonDimensionalVariable<T>(string componentName, string fieldName) where T : struct
        {
            FieldKey key = new FieldKey(componentName, fieldName);
            Dictionary<FieldKey, FieldValue<T?>> d;

            if (_nonDimensionalVariables.ContainsKey(typeof(T)))
            {
                d = (Dictionary<FieldKey, FieldValue<T?>>)_nonDimensionalVariables[typeof(T)];
            }
            else
            {
                d = new Dictionary<FieldKey, FieldValue<T?>>();
                _nonDimensionalVariables.Add(typeof(T), d);
            }

            if (_variablesKeysAlreadyUsed.Contains(key))
            {
                throw new ArgumentException();
            }

            d.Add(key, new FieldValue<T?>());

            _variablesKeysAlreadyUsed.Add(key);

            FieldValue<T?> v = d[key];

            _nonDimensionalVariablesGetterDelegates.Add(key, (NonDimensionalFieldGetter<T>)delegate { return (T)v.Value; });
            _nonDimensionalVariablesSetterDelegates.Add(key, (NonDimensionalFieldSetter<T>)delegate(T value) { v.Value = value; });
        }

        public void AddNonDimensionalVariable<T>(string componentName, string fieldName, T value) where T : struct
        {
            AddNonDimensionalVariable<T>(componentName, fieldName);

            Dictionary<FieldKey, FieldValue<T?>> d = (Dictionary<FieldKey, FieldValue<T?>>)_nonDimensionalVariables[typeof(T)];

            d[new FieldKey(componentName, fieldName)].Value = value;
        }

        public object GetNonDimensionalVariableGetter(string componentName, string fieldName)
        {
            return _nonDimensionalVariablesGetterDelegates[new FieldKey(componentName, fieldName)];
        }

        public NonDimensionalFieldGetter<T> GetNonDimensionalVariableGetter<T>(string componentName, string fieldname)
        {
            return (NonDimensionalFieldGetter<T>)GetNonDimensionalVariableGetter(componentName, fieldname);
        }

        public object GetNonDimensionalVariableSetter(string componentName, string fieldName)
        {
            return _nonDimensionalVariablesSetterDelegates[new FieldKey(componentName, fieldName)];
        }

        public NonDimensionalFieldSetter<T> GetNonDimensionalVariableSetter<T>(string componentName, string fieldName)
        {
            return (NonDimensionalFieldSetter<T>)GetNonDimensionalVariableSetter(componentName, fieldName);
        }

        public void SetNonDimensionalFieldValue<T>(string componentName, string fieldName, T value)
        {
            NonDimensionalFieldSetter<T> setter = GetNonDimensionalVariableSetter<T>(componentName, fieldName);
            setter(value);
        }

        #endregion

        #region DimensionalVariables

        Dictionary<FieldKey, object> _dimensionalVariables = new Dictionary<FieldKey, object>();

        public void Add1DimensionalVariable<D1, T>(string componentName, string fieldName)
        {
            FieldKey key = new FieldKey(componentName, fieldName);

            if (_variablesKeysAlreadyUsed.Contains(key))
            {
                throw new ArgumentException();
            }

            object v;
            if (typeof(D1) == typeof(Timestep))
            {
                v = new VariableWithTimeDimension<T>(this);
            }
            else
            {
                v = new Variable1Dimensional<D1, T>(this);
            }

            _dimensionalVariables.Add(key, v);
            _variablesKeysAlreadyUsed.Add(key);
        }


        public void Add2DimensionalVariable<D1, D2, T>(string componentName, string fieldName)
        {
            FieldKey key = new FieldKey(componentName, fieldName);

            if (_variablesKeysAlreadyUsed.Contains(key))
            {
                throw new ArgumentException();
            }

            object v;
            if (typeof(D1) == typeof(Timestep))
            {
                v = new Variable2DimensionalTime<D2, T>(this);
            }
            else
            {
                v = new Variable2Dimensional<D1, D2, T>(this);
            }

            _dimensionalVariables.Add(key, v);
            _variablesKeysAlreadyUsed.Add(key);
        }

        public void Add1DimensionalParameter<D1, T>(string componentName, string fieldName, IParameter1Dimensional<D1, T> parameter)
        {
            FieldKey key = new FieldKey(componentName, fieldName);

            if (_variablesKeysAlreadyUsed.Contains(key))
            {
                throw new ArgumentException();
            }

            _dimensionalVariables.Add(key, parameter);
            _variablesKeysAlreadyUsed.Add(key);
        }

        public void Add1DimensionalParameter<D1,T>(string componentName, string fieldName, Func<D1,T> parameter)
        {
            Add1DimensionalParameter(componentName, fieldName, new DelegateToParameter1Dimensional<D1,T>(parameter));
        }

        public void Add2DimensionalParameter<D1, D2, T>(string componentName, string fieldName, IParameter2Dimensional<D1, D2, T> parameter)
        {
            FieldKey key = new FieldKey(componentName, fieldName);

            if (_variablesKeysAlreadyUsed.Contains(key))
            {
                throw new ArgumentException();
            }

            _dimensionalVariables.Add(key, parameter);
            _variablesKeysAlreadyUsed.Add(key);
        }

        public void Add2DimensionalParameter<D1, D2, T>(string componentName, string fieldName, Func<D1,D2,T> parameter)
        {
            Add2DimensionalParameter(componentName, fieldName, new DelegateToParameter2Dimensional<D1, D2, T>(parameter, this));
        }

        public void ConnectParameterToVariable(string parameterComponentName, string parameterFieldName, string variableComponentName, string variableFieldName)
        {
            FieldKey key = new FieldKey(parameterComponentName, parameterFieldName);

            if (_variablesKeysAlreadyUsed.Contains(key))
            {
                throw new ArgumentException();
            }

            if (_dimensionalVariables.ContainsKey(new FieldKey(variableComponentName, variableFieldName)))
            {
                _dimensionalVariables.Add(key, _dimensionalVariables[new FieldKey(variableComponentName, variableFieldName)]);
            }
            else
            {
                throw new InvalidOperationException();
            }
            _variablesKeysAlreadyUsed.Add(key);
        }

        public void ConnectParameterToVariableNonDimensional<T>(string parameterComponentName, string parameterFieldName, string variableComponentName, string variableFieldName) where T : struct
        {
            FieldKey key = new FieldKey(parameterComponentName, parameterFieldName);
            FieldKey sourceKey = new FieldKey(variableComponentName, variableFieldName);

            if (_variablesKeysAlreadyUsed.Contains(key))
            {
                throw new ArgumentException();
            }

            Dictionary<FieldKey, FieldValue<T?>> d = (Dictionary<FieldKey, FieldValue<T?>>)_nonDimensionalVariables[typeof(T)];

            _variablesKeysAlreadyUsed.Add(key);

            FieldValue<T?> v = d[sourceKey];

            _nonDimensionalVariablesGetterDelegates.Add(key, (NonDimensionalFieldGetter<T>)delegate { return (T)v.Value; });
        }


        public object GetDimensionalField(string componentName, string fieldName)
        {
            FieldKey key = new FieldKey(componentName, fieldName);
            if(!_dimensionalVariables.ContainsKey(key))
            {
                throw new ArgumentOutOfRangeException(String.Format("{0}.{1} is not in the list of dimensional fields", componentName, fieldName));
            }

            return _dimensionalVariables[key];
        }

        public void Set1DimensionalParameter<D1, T>(string componentName, string fieldName, Func<D1, T> parameter)
        {
            var fieldKey = new FieldKey(componentName, fieldName);
            if (!_dimensionalVariables.ContainsKey(fieldKey))
            {
                throw new InvalidOperationException();
            }

            _dimensionalVariables[fieldKey] = new DelegateToParameter1Dimensional<D1, T>(parameter);
        }

        public void Set2DimensionalParameter<D1, D2, T>(string componentName, string fieldName, Func<D1, D2, T> parameter)
        {
            var fieldKey=new FieldKey(componentName, fieldName);
            if (!_dimensionalVariables.ContainsKey(fieldKey))
            {
                throw new InvalidOperationException();
            }

            _dimensionalVariables[fieldKey] = new DelegateToParameter2Dimensional<D1, D2, T>(parameter, this);
        }

        public void Set2DimensionalParameter<D1, D2, T>(string componentName, string fieldName, IParameter2Dimensional<D1, D2, T> parameter)
        {
            var fieldKey = new FieldKey(componentName, fieldName);
            if (!_dimensionalVariables.ContainsKey(fieldKey))
            {
                throw new InvalidOperationException();
            }

            _dimensionalVariables[fieldKey] = parameter;
        }


        #endregion

        #region Attached dimensions

        private List<string> _attachedDimensions = new List<string>();

        public IList<string> AttachedDimensions
        {
            get
            {
                return _attachedDimensions;
            }
        }

        #endregion

        public IEnumerable<Field> GetDimensionalFieldsOperator()
        {
            foreach (var f in _dimensionalVariables.Keys)
            {
                yield return new Field() { ComponentName = f.ComponentName, FieldName = f.FieldName, Values = _dimensionalVariables[f] };
            }
        }
    }
}
