﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using Esmf;
using System;

namespace Fund
{

    public class FundWorkflow
    {
        private Esmf.ModelFields _modelFields = new Esmf.ModelFields();
        private FundParameter _parameters;

        public Fund.Components.SocioEconomic.ISocioEconomicState SocioEconomicState;
        public Fund.Components.Emissions.IEmissionsState EmissionsState;
        public Fund.Components.Welfare.IGlobalWelfareState GlobalWelfareState;
        public Fund.Components.Welfare.IRegionalWelfareState RegionalWelfareState;
        public Fund.Components.ImpactAggregation.IImpactAggregationState ImpactAggregationState;
        public Fund.Components.Climate.IClimateState ClimateState;
        public Fund.Components.ClimateRegional.IClimateRegionalState ClimateRegionalState;
        public Fund.Components.Ocean.IOceanState OceanState;
        public Fund.Components.BioDiversity.IBioDiversityState BioDiversityState;
        public Fund.Components.ImpactBioDiversity.IImpactBioDiversityState ImpactBioDiversityState;
        public Fund.IFundOld2Climate FundOld2ClimateState;
        public Fund.ISocioEconomic2Welfare FundOld2GlobalWelfareState;
        public Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState ScenarioUncertaintyState;
        public Fund.Components.ImpactHeating.IImpactHeatingState ImpactHeatingState;
        public Fund.Components.ImpactCooling.IImpactCoolingState ImpactCoolingState;
        public Fund.Components.ImpactAgriculture.IImpactAgricultureState ImpactAgricultureState;
        public Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState GlobalHotellingTaxationState;
        public Fund.Components.ImpactWaterResources.IImpactWaterResourcesState ImpactWaterResourcesState;
        public Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState ImpactDiarrhoeaState;
        public Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState ImpactTropicalStormsState;
        public Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState ImpactExtratropicalStormsState;
        public Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState ImpactSeaLevelRiseState;
        public Fund.Components.Geography.IGeographyState GeographyState;
        public Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState TerrestrialBiosphereState;
        public Fund.Components.ImpactForests.IImpactForestsState ImpactForestsState;
        public Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState ImpactVectorBorneDiseases;
        public Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState ImpactCardiovascularRespiratoryState;
        public Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState ImpactDeathMorbidityState;

        public Esmf.ModelFields ModelFields { get { return _modelFields; } }

        public void Run(ModelOutput output)
        {
            ReCreateStateVariables();

            var clock = new Clock(Timestep.FromSimulationYear(0), Timestep.FromSimulationYear(LegacyConstants.NYear));

            RunScenarioUncertaintyComponent(clock);
            RunGeographyComponent(clock);
            RunImpactSeaLevelRiseComponent(clock);
            RunTerrestrialBiosphereComponent(clock);
            RunSocioEconomicComponent(clock);

            clock.Advance();

            while (!clock.IsDone)
            {
                RunScenarioUncertaintyComponent(clock);

                RunGeographyComponent(clock);

                RunSocioEconomicComponent(clock);

                RunGlobalHotellingTaxationComponent(clock);

                RunEmissionsComponent(clock);

                RunTerrestrialBiosphereComponent(clock);

                RunFundOld2ClimateState(clock);
                RunClimateComponent(clock);
                RunClimateRegionalComponent(clock);

                RunOceanComponent(clock);

                RunBioDiversityComponent(clock);

                RunImpactBioDiversityComponent(clock);

                RunImpactHeatingComponent(clock);
                RunImpactCoolingComponent(clock);

                RunImpactAgricultureComponent(clock);

                RunImpactWaterResourcesComponent(clock);

                RunImpactDiarrhoeaComponent(clock);

                RunImpactTropicalStormsComponent(clock);

                RunImpactExtratropicalStormsComponent(clock);

                RunImpactSeaLevelRiseComponent(clock);

                RunImpactForestsComponent(clock);

                RunImpactVectorBorneDiseasesComponent(clock);

                RunImpactCardiovascularRespiratoryComponent(clock);

                RunImpactDeathMorbidityComponent(clock);

                foreach (Fund.CommonDimensions.Region r in Enum.GetValues(typeof(Fund.CommonDimensions.Region)))
                {

                    if (output != null)
                    {
                        var t = clock.Current;

                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sWater, -ImpactWaterResourcesState.water[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sForests, -ImpactForestsState.forests[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sHeating, -ImpactHeatingState.heating[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sCooling, -ImpactCoolingState.cooling[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sAgriculture, -ImpactAgricultureState.agcost[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sDryland, ImpactSeaLevelRiseState.drycost[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sSeaProtection, ImpactSeaLevelRiseState.protcost[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sImigration, ImpactSeaLevelRiseState.entercost[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sHurrican, ImpactTropicalStormsState.hurrdam[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sExtratropicalStorms, ImpactExtratropicalStormsState.extratropicalstormsdam[clock.Current, r] * 1000000000));

                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sSpecies, ImpactBioDiversityState.species[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sDeath, ImpactDeathMorbidityState.deadcost[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sMorbidity, ImpactDeathMorbidityState.morbcost[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sWetland, ImpactSeaLevelRiseState.wetcost[clock.Current, r] * 1000000000));
                        output.Damages.Add(new Damage(t.Value, (int)r, Sector.sEmigration, ImpactSeaLevelRiseState.leavecost[clock.Current, r] * 1000000000));

                        // Add GDP for that year and region to the Output object
                        // income is multiplied by 1 billion, since that is the unit
                        // of income
                        output.Incomes.Add(clock.Current.Value, (int)r, SocioEconomicState.income[clock.Current, r] * 1000000000);

                        // Add population for that year and region to the Output object
                        // population is multiplied by 1 million, since that is the unit of
                        // population
                        output.Populations.Add(clock.Current.Value, (int)r, SocioEconomicState.population[clock.Current, r] * 1000000);
                    }
                }

                RunImpactAggregationComponent(clock);

                RunFundOld2GlobalWelfareState(clock);
                RunGlobalWelfareComponent(clock);

                RunRegionalWelfareComponent(clock);

                clock.Advance();
            }
        }

        public void RunSocioEconomicComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var lSocioEconomicComponent = new Fund.Components.SocioEconomic.SocioEconomicComponent();

            // Run SocioEconomic component
            lSocioEconomicComponent.Run(clock, SocioEconomicState);
        }

        public void RunClimateComponent(Clock clock)
        {
            var t = clock.Current;

            // Set up climate component
            var lclimateComp = new Fund.Components.Climate.ClimateComponent();

            // Run climate component
            lclimateComp.Run(clock, ClimateState);

        }

        public void InitClimateState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("climate", "ClimateSensitivity", parameters.ClimateSensitivity);
            _modelFields.AddNonDimensionalVariable<double>("climate", "so2ind", 0.08);
            _modelFields.AddNonDimensionalVariable<double>("climate", "so2dir", 0.03);
            _modelFields.AddNonDimensionalVariable<double>("climate", "sf6pre", 0.04);
            _modelFields.AddNonDimensionalVariable<double>("climate", "lifesf6", parameters.lifesf6);
            _modelFields.AddNonDimensionalVariable<double>("climate", "n2opre", 285);
            _modelFields.AddNonDimensionalVariable<double>("climate", "lifen2o", parameters.lifen2o);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2pre", 275);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac5", 0.10);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac4", 0.25);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac3", 0.32);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac2", 0.20);
            _modelFields.AddNonDimensionalVariable<double>("climate", "co2frac1", 0.13);
            _modelFields.AddNonDimensionalVariable<double>("climate", "lifeco1", parameters.lifeco1);
            _modelFields.AddNonDimensionalVariable<double>("climate", "lifeco2", parameters.lifeco2);
            _modelFields.AddNonDimensionalVariable<double>("climate", "lifeco3", parameters.lifeco3);
            _modelFields.AddNonDimensionalVariable<double>("climate", "lifeco4", parameters.lifeco4);
            _modelFields.AddNonDimensionalVariable<double>("climate", "lifeco5", parameters.lifeco5);
            _modelFields.AddNonDimensionalVariable<double>("climate", "ch4pre", 790);
            _modelFields.AddNonDimensionalVariable<double>("climate", "lifech4", parameters.lifech4);
            _modelFields.Add1DimensionalParameter<Esmf.Timestep, double>("climate", "RunAwayCH4", new YearParameterConnector(parameters, "RunAwayCH4"));
            _modelFields.AddNonDimensionalVariable<double>("climate", "ch4add", parameters.ch4add);
            _modelFields.AddNonDimensionalVariable<double>("climate", "n2oadd", parameters.n2oadd);
            _modelFields.AddNonDimensionalVariable<double>("climate", "sf6add", parameters.sf6add);
            _modelFields.AddNonDimensionalVariable<bool>("climate", "addmp", false);
            _modelFields.AddNonDimensionalVariable<double>("climate", "LifeTempConst", parameters.LifeTempConst);
            _modelFields.AddNonDimensionalVariable<double>("climate", "LifeTempLin", parameters.LifeTempLin);

            // Parameters coming from different components, need to be coupled
            //_modelFields.ConnectParameterToVariableNonDimensional<double>("climate","ClimateSensitivity","roebakerclimatesensitivity", "ClimateSensitivity");
            _modelFields.ConnectParameterToVariable("climate", "mco2", "emissions", "mco2");
            _modelFields.ConnectParameterToVariable("climate", "globso2", "emissions", "globso2");
            _modelFields.ConnectParameterToVariable("climate", "globsf6", "emissions", "globsf6");
            _modelFields.ConnectParameterToVariable("climate", "globn2o", "emissions", "globn2o");
            _modelFields.ConnectParameterToVariable("climate", "globch4", "emissions", "globch4");
            _modelFields.ConnectParameterToVariable("climate", "addco2", "fundold2climate", "addco2");
            _modelFields.ConnectParameterToVariable("climate", "addch4", "fundold2climate", "addch4");
            _modelFields.ConnectParameterToVariable("climate", "addn2o", "fundold2climate", "addn2o");
            _modelFields.ConnectParameterToVariable("climate", "addsf6", "fundold2climate", "addsf6");
            _modelFields.ConnectParameterToVariable("climate", "TerrestrialCO2", "terrestrialbiosphere", "TerrestrialCO2");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Climate.IClimateState));
            ClimateState = ss.ConnectToState<Fund.Components.Climate.IClimateState>(_modelFields, "climate");

            // Initial values, should eventually also come from parameter file
            ClimateState.temp[Timestep.FromSimulationYear(0)] = 0.20;
            ClimateState.cbox1[Timestep.FromSimulationYear(0)] = 283.53;
            ClimateState.cbox2[Timestep.FromSimulationYear(0)] = 5.62;
            ClimateState.cbox3[Timestep.FromSimulationYear(0)] = 6.29;
            ClimateState.cbox4[Timestep.FromSimulationYear(0)] = 2.19;
            ClimateState.cbox5[Timestep.FromSimulationYear(0)] = 0.15;
            ClimateState.acco2[Timestep.FromSimulationYear(0)] = ClimateState.cbox1[Timestep.FromSimulationYear(0)]
                + ClimateState.cbox2[Timestep.FromSimulationYear(0)]
                + ClimateState.cbox3[Timestep.FromSimulationYear(0)]
                + ClimateState.cbox4[Timestep.FromSimulationYear(0)]
                + ClimateState.cbox5[Timestep.FromSimulationYear(0)];
            ClimateState.acsf6[Timestep.FromSimulationYear(0)] = ClimateState.sf6pre;
            ClimateState.acn2o[Timestep.FromSimulationYear(0)] = 296;
            ClimateState.acch4[Timestep.FromSimulationYear(0)] = 1222;
            ClimateState.maxrf = 0;
        }

        public void InitSocioEconomicState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("socioeconomic", "consleak", parameters.consleak);
            _modelFields.AddNonDimensionalVariable<bool>("socioeconomic", "runwithoutdamage", false);
            _modelFields.AddNonDimensionalVariable<bool>("socioeconomic", "runwithoutpopulationperturbation", false);
            _modelFields.AddNonDimensionalVariable<double>("socioeconomic", "plusel", parameters.plusel);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("socioeconomic", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("socioeconomic", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("socioeconomic", "gdp0", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp0"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("socioeconomic", "pop0", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop0"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("socioeconomic", "plus90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "plus90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("socioeconomic", "urbcorr", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "urbcorr"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("socioeconomic", "ypcgrowth", "scenariouncertainty", "ypcgrowth");
            _modelFields.ConnectParameterToVariable("socioeconomic", "pgrowth", "scenariouncertainty", "pgrowth");

            // Parameters coming from different components, need to be coupled
            _modelFields.ConnectParameterToVariable("socioeconomic", "leave", "impactsealevelrise", "leave");
            _modelFields.ConnectParameterToVariable("socioeconomic", "enter", "impactsealevelrise", "enter");
            _modelFields.ConnectParameterToVariable("socioeconomic", "eloss", "impactaggregation", "eloss");
            _modelFields.ConnectParameterToVariable("socioeconomic", "sloss", "impactaggregation", "sloss");
            _modelFields.ConnectParameterToVariable("socioeconomic", "dead", "impactdeathmorbidity", "dead");
            _modelFields.ConnectParameterToVariable("socioeconomic", "mitigationcost", "emissions", "mitigationcost");
            _modelFields.ConnectParameterToVariable("socioeconomic", "area", "geography", "area");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.SocioEconomic.ISocioEconomicState));
            SocioEconomicState = ss.ConnectToState<Fund.Components.SocioEconomic.ISocioEconomicState>(_modelFields, "socioeconomic");
        }

        public FundWorkflow(FundParameter parameters)
        {
            _parameters = parameters;

            InitVariables(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState), "scenariouncertainty");
            InitVariables(typeof(Fund.Components.BioDiversity.IBioDiversityState), "biodiversity");
            InitVariables(typeof(Fund.Components.Climate.IClimateState), "climate");
            InitVariables(typeof(Fund.Components.ClimateRegional.IClimateRegionalState), "climateregional");
            InitVariables(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState), "impactbiodiversity");
            InitVariables(typeof(Fund.Components.ImpactHeating.IImpactHeatingState), "impactheating");
            InitVariables(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState), "impactagriculture");
            InitVariables(typeof(Fund.Components.ImpactCooling.IImpactCoolingState), "impactcooling");
            InitVariables(typeof(Fund.Components.Ocean.IOceanState), "ocean");
            InitVariables(typeof(Fund.Components.SocioEconomic.ISocioEconomicState), "socioeconomic");
            InitVariables(typeof(Fund.Components.Emissions.IEmissionsState), "emissions");
            InitVariables(typeof(Fund.Components.Welfare.IGlobalWelfareState), "globalwelfare");
            InitVariables(typeof(Fund.Components.Welfare.IRegionalWelfareState), "regionalwelfare");
            InitVariables(typeof(Fund.IFundOld2Climate), "fundold2climate");
            InitVariables(typeof(Fund.ISocioEconomic2Welfare), "fundold2globalwelfare");
            InitVariables(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState), "impactaggregation");
            InitVariables(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState), "globalhotellingtaxation");
            InitVariables(typeof(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState), "impactwaterresources");
            InitVariables(typeof(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState), "impactdiarrhoea");
            InitVariables(typeof(Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState), "impacttropicalstorms");
            InitVariables(typeof(Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState), "impactextratropicalstorms");
            InitVariables(typeof(Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState), "impactsealevelrise");
            InitVariables(typeof(Fund.Components.Geography.IGeographyState), "geography");
            InitVariables(typeof(Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState), "terrestrialbiosphere");
            InitVariables(typeof(Fund.Components.ImpactForests.IImpactForestsState), "impactforests");
            InitVariables(typeof(Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState), "impactvectorbornediseases");
            InitVariables(typeof(Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState), "impactcardiovascularrespiratory");
            InitVariables(typeof(Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState), "impactdeathmorbidity");

            InitScenarioUncertaintyState(_parameters);
            InitClimateState(_parameters);
            InitClimateRegionalState(_parameters);
            InitOceanState(_parameters);
            InitBioDiversityState(_parameters);
            InitSocioEconomicState(_parameters);
            InitEmissionsState(_parameters);
            InitGlobalWelfareState(_parameters);
            InitRegionalWelfareState(_parameters);
            InitImpactBioDiversityState(_parameters);
            InitImpactHeatingState(_parameters);
            InitImpactAgricultureState(_parameters);
            InitImpactCoolingState(_parameters);
            InitFundOld2ClimateState(_parameters);
            InitFundOld2GlobalWelfare(_parameters);
            InitImpactAggregationState(_parameters);
            InitGlobalHotellingTaxationState(_parameters);
            InitImpactWaterResourcesState(_parameters);
            InitImpactDiarrhoeaState(_parameters);
            InitImpactTropicalStormsState(_parameters);
            InitImpactExtratropicalStormsState(_parameters);
            InitImpactSeaLevelRiseState(_parameters);
            InitGeographyState(_parameters);
            InitTerrestrialBiosphereState(_parameters);
            InitImpactForestsState(_parameters);
            InitImpactVectorBorneDiseasesState(_parameters);
            InitImpactCardiovascularRespiratoryState(_parameters);
            InitImpactDeathMorbidityState(_parameters);
        }

        public void InitOceanState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("ocean", "lifesea", parameters.lifesea);
            _modelFields.AddNonDimensionalVariable<double>("ocean", "seas", parameters.seas);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("ocean", "temp", "climate", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Ocean.IOceanState));
            OceanState = ss.ConnectToState<Fund.Components.Ocean.IOceanState>(_modelFields, "ocean");

            // Initial values, should eventually also come from parameter file
            var t0 = Timestep.FromSimulationYear(0);
            OceanState.sea[t0] = 0;
        }

        public void RunOceanComponent(Clock clock)
        {
            var t = clock.Current;

            // Set up ocean component
            var oc = new Fund.Components.Ocean.OceanComponent();

            // Run ocean component
            oc.Run(clock, OceanState);
        }

        public void InitBioDiversityState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("biodiversity", "nospecbase", parameters.nospecbase);
            _modelFields.AddNonDimensionalVariable<double>("biodiversity", "bioloss", parameters.bioloss);
            _modelFields.AddNonDimensionalVariable<double>("biodiversity", "biosens", parameters.biosens);
            _modelFields.AddNonDimensionalVariable<double>("biodiversity", "dbsta", parameters.DBsTa);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("biodiversity", "temp", "climate", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.BioDiversity.IBioDiversityState));
            BioDiversityState = ss.ConnectToState<Fund.Components.BioDiversity.IBioDiversityState>(_modelFields, "biodiversity");

            // Initial values, should eventually also come from parameter file
            // Carefull, the initial value is for the year 50!
            BioDiversityState.nospecies[Timestep.FromSimulationYear(0)] = BioDiversityState.nospecbase;
        }

        public void RunBioDiversityComponent(Clock clock)
        {
            // Set up biodiv component
            var bc = new Fund.Components.BioDiversity.BioDiversityComponent();

            // Run biodiv component
            bc.Run(clock, BioDiversityState);
        }

        public void InitImpactBioDiversityState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactbiodiversity", "nospecbase", parameters.nospecbase);
            _modelFields.AddNonDimensionalVariable<double>("impactbiodiversity", "bioshare", parameters.bioshare);
            _modelFields.AddNonDimensionalVariable<double>("impactbiodiversity", "spbm", parameters.spbm);
            _modelFields.AddNonDimensionalVariable<double>("impactbiodiversity", "valbase", parameters.valbase);
            _modelFields.AddNonDimensionalVariable<double>("impactbiodiversity", "dbsta", parameters.DBsTa);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactbiodiversity", "valinc", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "valinc"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactbiodiversity", "temp", "climateregional", "temp");
            _modelFields.ConnectParameterToVariable("impactbiodiversity", "nospecies", "biodiversity", "nospecies");

            // Parameters coming from different components, explicitly coupled
            _modelFields.ConnectParameterToVariable("impactbiodiversity", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactbiodiversity", "population", "socioeconomic", "population");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState));
            ImpactBioDiversityState = ss.ConnectToState<Fund.Components.ImpactBioDiversity.IImpactBioDiversityState>(_modelFields, "impactbiodiversity");

        }

        public void RunImpactBioDiversityComponent(Clock clock)
        {
            // Set up ImpactBioDiversity component
            var lImpactBioDiversityComponent = new Fund.Components.ImpactBioDiversity.ImpactBioDiversityComponent();

            // Run ImpactBioDiversity component
            lImpactBioDiversityComponent.Run(clock, ImpactBioDiversityState);
        }

        public void InitVariables(System.Type stateinterface, string componentName)
        {
            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(stateinterface);
            ss.CreateVariables(_modelFields, componentName);
        }

        public void InitFundOld2ClimateState(FundParameter parameters)
        {
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2Climate));
            FundOld2ClimateState = ss.ConnectToState<Fund.IFundOld2Climate>(_modelFields, "fundold2climate");
        }

        public void RunFundOld2ClimateState(Clock clock)
        {
            var c = new Fund.FundOld2ClimateComponent();

            c.Run(clock, FundOld2ClimateState, _parameters);
        }

        public void InitGlobalWelfareState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("globalwelfare", "prtp", 0.03);
            _modelFields.AddNonDimensionalVariable<double>("globalwelfare", "elasticityofmarginalutility", 1.0);
            _modelFields.AddNonDimensionalVariable<double>("globalwelfare", "utilitycalibrationadditive", 0.0);
            _modelFields.AddNonDimensionalVariable<double>("globalwelfare", "utilitycalibrationmultiplicative", 1.0);
            _modelFields.AddNonDimensionalVariable<Timestep>("globalwelfare", "starttimestep", Timestep.FromYear(2000));


            // Parameters coming from different components, need to be coupled
            _modelFields.ConnectParameterToVariable("globalwelfare", "consumption", "fundold2globalwelfare", "totalconsumption");
            _modelFields.ConnectParameterToVariable("globalwelfare", "population", "fundold2globalwelfare", "totalpopulation");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IGlobalWelfareState));
            GlobalWelfareState = ss.ConnectToState<Fund.Components.Welfare.IGlobalWelfareState>(_modelFields, "globalwelfare");

            // Initial values, should eventually also come from parameter file
            var t0 = Timestep.FromSimulationYear(0);
            //WelfareState.welfare[t0]=0;
        }

        public void InitFundOld2GlobalWelfare(FundParameter parameters)
        {
            // Parameters coming from different components, need to be coupled
            _modelFields.ConnectParameterToVariable("fundold2globalwelfare", "consumption", "socioeconomic", "consumption");
            _modelFields.ConnectParameterToVariable("fundold2globalwelfare", "population", "socioeconomic", "population");


            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2Welfare));
            FundOld2GlobalWelfareState = ss.ConnectToState<Fund.ISocioEconomic2Welfare>(_modelFields, "fundold2globalwelfare");
        }

        public void RunFundOld2GlobalWelfareState(Clock clock)
        {
            var c = new Fund.SocioEconomic2Welfare();

            c.Run(clock, FundOld2GlobalWelfareState);
        }

        public void RunGlobalWelfareComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var lComponent = new Fund.Components.Welfare.GlobalWelfareComponent();

            // Run SocioEconomic component
            lComponent.Run(clock, GlobalWelfareState);
        }

        public void InitRegionalWelfareState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("regionalwelfare", "prtp", 0.03);
            _modelFields.AddNonDimensionalVariable<double>("regionalwelfare", "elasticityofmarginalutility", 1.0);
            _modelFields.AddNonDimensionalVariable<double>("regionalwelfare", "utilitycalibrationadditive", 0.0);
            _modelFields.AddNonDimensionalVariable<double>("regionalwelfare", "utilitycalibrationmultiplicative", 1.0);
            _modelFields.AddNonDimensionalVariable<Timestep>("regionalwelfare", "starttimestep", Timestep.FromYear(2000));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, Double>("regionalwelfare", "welfareweight",
                (Timestep t, Fund.CommonDimensions.Region r) => 1.0);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("regionalwelfare", "consumption", "socioeconomic", "consumption");

            // Parameters coming from different components, need to be coupled
            _modelFields.ConnectParameterToVariable("regionalwelfare", "population", "socioeconomic", "populationin1");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IRegionalWelfareState));
            RegionalWelfareState = ss.ConnectToState<Fund.Components.Welfare.IRegionalWelfareState>(_modelFields, "regionalwelfare");

        }

        public void RunRegionalWelfareComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var lComponent = new Fund.Components.Welfare.RegionalWelfareComponent();

            // Run SocioEconomic component
            lComponent.Run(clock, RegionalWelfareState);
        }

        public void InitScenarioUncertaintyState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<Timestep>("scenariouncertainty", "timeofuncertaintystart", Timestep.FromYear((int) parameters.timeofuncertaintystart));

            // Parameters coming from file
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputypcgrowth", new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ypcgrowth"));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputpgrowth", new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pgrowth"));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputaeei", new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "AEEI"));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputacei", new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ACEI"));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, double>("scenariouncertainty", "inputForestEmm", new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ForestEmm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("scenariouncertainty", "ecgradd", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ecgradd"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("scenariouncertainty", "pgadd", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pgadd"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("scenariouncertainty", "aeeiadd", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "aeeiadd"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("scenariouncertainty", "aceiadd", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "aceiadd"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("scenariouncertainty", "foremadd", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "foremadd"));

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState));
            ScenarioUncertaintyState = ss.ConnectToState<Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState>(_modelFields, "scenariouncertainty");
        }

        public void RunScenarioUncertaintyComponent(Clock clock)
        {
            var c = new Fund.Components.ScenarioUncertainty.ScenarioUncertaintyComponent();

            c.Run(clock, ScenarioUncertaintyState);
        }

        public void InitImpactHeatingState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactheating", "heel", parameters.heel);
            _modelFields.AddNonDimensionalVariable<double>("impactheating", "henl", parameters.henl);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactheating", "hebm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "hebm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactheating", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactheating", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactheating", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactheating", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactheating", "temp", "climateregional", "temp");

            // Parameters coming from different components, explicitly coupled
            _modelFields.ConnectParameterToVariable("impactheating", "cumaeei", "emissions", "cumaeei");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactHeating.IImpactHeatingState));
            ImpactHeatingState = ss.ConnectToState<Fund.Components.ImpactHeating.IImpactHeatingState>(_modelFields, "impactheating");
        }

        public void RunImpactHeatingComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactHeating.ImpactHeatingComponent();

            // Run ImpactBioDiversity component
            c.Run(clock, ImpactHeatingState);
        }

        public void InitImpactCoolingState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactcooling", "ceel", parameters.ceel);
            _modelFields.AddNonDimensionalVariable<double>("impactcooling", "cenl", parameters.cenl);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcooling", "cebm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "cebm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcooling", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcooling", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactcooling", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactcooling", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactcooling", "temp", "climateregional", "temp");

            // Parameters coming from different components, explicitly coupled
            _modelFields.ConnectParameterToVariable("impactcooling", "cumaeei", "emissions", "cumaeei");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCooling.IImpactCoolingState));
            ImpactCoolingState = ss.ConnectToState<Fund.Components.ImpactCooling.IImpactCoolingState>(_modelFields, "impactcooling");
        }

        public void RunImpactCoolingComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactCooling.ImpactCoolingComponent();

            // Run ImpactBioDiversity component
            c.Run(clock, ImpactCoolingState);
        }

        public void InitImpactAgricultureState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactagriculture", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactagriculture", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactagriculture", "agrish0", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "agrish0"));
            _modelFields.AddNonDimensionalVariable<double>("impactagriculture", "agel", parameters.agel);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactagriculture", "agrbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "agrbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactagriculture", "agtime", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "agtime"));
            _modelFields.AddNonDimensionalVariable<double>("impactagriculture", "agnl", parameters.agnl);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactagriculture", "aglbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "aglbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactagriculture", "agopt", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "agopt"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactagriculture", "agcbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "agcbm"));
            _modelFields.AddNonDimensionalVariable<double>("impactagriculture", "co2pre", parameters.co2pre);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactagriculture", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactagriculture", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactagriculture", "temp", "climateregional", "temp");
            _modelFields.ConnectParameterToVariable("impactagriculture", "acco2", "climate", "acco2");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState));
            ImpactAgricultureState = ss.ConnectToState<Fund.Components.ImpactAgriculture.IImpactAgricultureState>(_modelFields, "impactagriculture");

            // Initial values, should eventually also come from parameter file
            var t0 = Timestep.FromSimulationYear(0);
            foreach (Fund.CommonDimensions.Region r in Enum.GetValues(typeof(Fund.CommonDimensions.Region)))
            {
                ImpactAgricultureState.agrate[t0, r] = ImpactAgricultureState.agrbm[r] * Math.Pow(0.005 / LegacyConstants.DBsT, ImpactAgricultureState.agnl) * ImpactAgricultureState.agtime[r];
            }
        }

        public void RunImpactAgricultureComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactAgriculture.ImpactAgricultureComponent();

            // Run ImpactBioDiversity component
            c.Run(clock, ImpactAgricultureState);
        }

        public void InitImpactAggregationState(FundParameter parameters)
        {
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffwater", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffforests", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffheating", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffcooling", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffagcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffdrycost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffprotcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffentercost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffhurrdam", false);
            // tropical storms switched off while in development
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffextratropicalstormsdam", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffspecies", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffdeadcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffmorbcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffwetcost", false);
            _modelFields.AddNonDimensionalVariable<bool>("impactaggregation", "switchoffleavecost", false);

            _modelFields.ConnectParameterToVariable("impactaggregation", "income", "socioeconomic", "income");

            _modelFields.ConnectParameterToVariable("impactaggregation", "heating", "impactheating", "heating");
            _modelFields.ConnectParameterToVariable("impactaggregation", "cooling", "impactcooling", "cooling");
            _modelFields.ConnectParameterToVariable("impactaggregation", "agcost", "impactagriculture", "agcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "species", "impactbiodiversity", "species");
            _modelFields.ConnectParameterToVariable("impactaggregation", "water", "impactwaterresources", "water");
            _modelFields.ConnectParameterToVariable("impactaggregation", "hurrdam", "impacttropicalstorms", "hurrdam");
            _modelFields.ConnectParameterToVariable("impactaggregation", "extratropicalstormsdam", "impactextratropicalstorms", "extratropicalstormsdam");

            _modelFields.ConnectParameterToVariable("impactaggregation", "forests", "impactforests", "forests");
            _modelFields.ConnectParameterToVariable("impactaggregation", "drycost", "impactsealevelrise", "drycost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "protcost", "impactsealevelrise", "protcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "entercost", "impactsealevelrise", "entercost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "deadcost", "impactdeathmorbidity", "deadcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "morbcost", "impactdeathmorbidity", "morbcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "wetcost", "impactsealevelrise", "wetcost");
            _modelFields.ConnectParameterToVariable("impactaggregation", "leavecost", "impactsealevelrise", "leavecost");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState));
            ImpactAggregationState = ss.ConnectToState<Fund.Components.ImpactAggregation.IImpactAggregationState>(_modelFields, "impactaggregation");

            // Initial values, should eventually also come from parameter file
            foreach (Fund.CommonDimensions.Region r in Enum.GetValues(typeof(Fund.CommonDimensions.Region)))
            {
                ImpactAggregationState.eloss[Timestep.FromSimulationYear(0), r] = 0.0;
                ImpactAggregationState.sloss[Timestep.FromSimulationYear(0), r] = 0.0;
            }
        }

        public void RunImpactAggregationComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactAggregation.ImpactAggregationComponent();

            c.Run(clock, ImpactAggregationState);
        }

        public void InitGlobalHotellingTaxationState(FundParameter parameters)
        {
            _modelFields.AddNonDimensionalVariable<double>("globalhotellingtaxation", "prtp", 0.03);
            _modelFields.AddNonDimensionalVariable<double>("globalhotellingtaxation", "elasticityofmarginalutility", 1.0);
            _modelFields.AddNonDimensionalVariable<double>("globalhotellingtaxation", "basetax", 0.0);
            _modelFields.AddNonDimensionalVariable<Timestep>("globalhotellingtaxation", "baseyear", Timestep.FromSimulationYear(50));

            _modelFields.ConnectParameterToVariable("globalhotellingtaxation", "consumption", "socioeconomic", "globalconsumption");
            _modelFields.ConnectParameterToVariable("globalhotellingtaxation", "population", "socioeconomic", "globalpopulation");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState));
            GlobalHotellingTaxationState = ss.ConnectToState<Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState>(_modelFields, "globalhotellingtaxation");

        }

        public void RunGlobalHotellingTaxationComponent(Clock clock)
        {
            var c = new Fund.Components.HotellingTaxation.GlobalHotellingTaxationComponent();

            c.Run(clock, GlobalHotellingTaxationState);

        }

        public void ReCreateStateVariables()
        {
            // Create proxy
            var s1 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.SocioEconomic.ISocioEconomicState));
            SocioEconomicState = s1.ConnectToState<Fund.Components.SocioEconomic.ISocioEconomicState>(_modelFields, "socioeconomic");

            var s2 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IGlobalWelfareState));
            GlobalWelfareState = s2.ConnectToState<Fund.Components.Welfare.IGlobalWelfareState>(_modelFields, "globalwelfare");

            var s3 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Welfare.IRegionalWelfareState));
            RegionalWelfareState = s3.ConnectToState<Fund.Components.Welfare.IRegionalWelfareState>(_modelFields, "regionalwelfare");

            var s4 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAggregation.IImpactAggregationState));
            ImpactAggregationState = s4.ConnectToState<Fund.Components.ImpactAggregation.IImpactAggregationState>(_modelFields, "impactaggregation");

            var s5 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Climate.IClimateState));
            ClimateState = s5.ConnectToState<Fund.Components.Climate.IClimateState>(_modelFields, "climate");

            var s6 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Ocean.IOceanState));
            OceanState = s6.ConnectToState<Fund.Components.Ocean.IOceanState>(_modelFields, "ocean");

            var s7 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.BioDiversity.IBioDiversityState));
            BioDiversityState = s7.ConnectToState<Fund.Components.BioDiversity.IBioDiversityState>(_modelFields, "biodiversity");

            var s8 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactBioDiversity.IImpactBioDiversityState));
            ImpactBioDiversityState = s8.ConnectToState<Fund.Components.ImpactBioDiversity.IImpactBioDiversityState>(_modelFields, "impactbiodiversity");

            var s12 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.IFundOld2Climate));
            FundOld2ClimateState = s12.ConnectToState<Fund.IFundOld2Climate>(_modelFields, "fundold2climate");

            var s13 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.ISocioEconomic2Welfare));
            FundOld2GlobalWelfareState = s13.ConnectToState<Fund.ISocioEconomic2Welfare>(_modelFields, "fundold2globalwelfare");

            var s15 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState));
            ScenarioUncertaintyState = s15.ConnectToState<Fund.Components.ScenarioUncertainty.IScenarioUncertaintyState>(_modelFields, "scenariouncertainty");

            var s16 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactHeating.IImpactHeatingState));
            ImpactHeatingState = s16.ConnectToState<Fund.Components.ImpactHeating.IImpactHeatingState>(_modelFields, "impactheating");

            var s17 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCooling.IImpactCoolingState));
            ImpactCoolingState = s17.ConnectToState<Fund.Components.ImpactCooling.IImpactCoolingState>(_modelFields, "impactcooling");

            var s19 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactAgriculture.IImpactAgricultureState));
            ImpactAgricultureState = s19.ConnectToState<Fund.Components.ImpactAgriculture.IImpactAgricultureState>(_modelFields, "impactagriculture");

            var s21 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState));
            GlobalHotellingTaxationState = s21.ConnectToState<Fund.Components.HotellingTaxation.IGlobalHotellingTaxationState>(_modelFields, "globalhotellingtaxation");

            var s22 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState));
            ImpactWaterResourcesState = s22.ConnectToState<Fund.Components.ImpactWaterResources.IImpactWaterResourcesState>(_modelFields, "impactwaterresources");

            var s23 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState));
            ImpactDiarrhoeaState = s23.ConnectToState<Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState>(_modelFields, "impactdiarrhoea");

            var s24 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ClimateRegional.IClimateRegionalState));
            ClimateRegionalState = s24.ConnectToState<Fund.Components.ClimateRegional.IClimateRegionalState>(_modelFields, "climateregional");

            var s25 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState));
            ImpactTropicalStormsState = s25.ConnectToState<Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState>(_modelFields, "impacttropicalstorms");

            var s26 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState));
            ImpactExtratropicalStormsState = s26.ConnectToState<Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState>(_modelFields, "impactextratropicalstorms");

            var s27 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState));
            ImpactSeaLevelRiseState = s27.ConnectToState<Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState>(_modelFields, "impactsealevelrise");

            var s28 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Geography.IGeographyState));
            GeographyState = s28.ConnectToState<Fund.Components.Geography.IGeographyState>(_modelFields, "geography");

            var s29 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Emissions.IEmissionsState));
            EmissionsState = s29.ConnectToState<Fund.Components.Emissions.IEmissionsState>(_modelFields, "emissions");

            var s30 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState));
            TerrestrialBiosphereState = s30.ConnectToState<Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState>(_modelFields, "terrestrialbiosphere");

            var s31 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactForests.IImpactForestsState));
            ImpactForestsState = s31.ConnectToState<Fund.Components.ImpactForests.IImpactForestsState>(_modelFields, "impactforests");

            var s32 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState));
            ImpactVectorBorneDiseases = s32.ConnectToState<Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState>(_modelFields, "impactvectorbornediseases");

            var s33 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState));
            ImpactCardiovascularRespiratoryState = s33.ConnectToState<Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState>(_modelFields, "impactcardiovascularrespiratory");

            var s34 = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState));
            ImpactDeathMorbidityState = s34.ConnectToState<Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState>(_modelFields, "impactdeathmorbidity");

        }

        public void InitImpactWaterResourcesState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactwaterresources", "watechrate", parameters.watechrate);
            _modelFields.AddNonDimensionalVariable<double>("impactwaterresources", "wrel", parameters.wrel);
            _modelFields.AddNonDimensionalVariable<double>("impactwaterresources", "wrnl", parameters.wrnl);
            _modelFields.AddNonDimensionalVariable<double>("impactwaterresources", "wrpl", parameters.wrpl);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactwaterresources", "wrbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "wrbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactwaterresources", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactwaterresources", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactwaterresources", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactwaterresources", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactwaterresources", "temp", "climateregional", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactWaterResources.IImpactWaterResourcesState));
            ImpactWaterResourcesState = ss.ConnectToState<Fund.Components.ImpactWaterResources.IImpactWaterResourcesState>(_modelFields, "impactwaterresources");
        }

        public void RunImpactWaterResourcesComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactWaterResources.ImpactWaterResourcesComponent();

            c.Run(clock, ImpactWaterResourcesState);
        }

        public void InitImpactDiarrhoeaState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactdiarrhoea", "diamortel", parameters.diamortel);
            _modelFields.AddNonDimensionalVariable<double>("impactdiarrhoea", "diamortnl", parameters.diamortnl);
            _modelFields.AddNonDimensionalVariable<double>("impactdiarrhoea", "diayldel", parameters.diayldel);
            _modelFields.AddNonDimensionalVariable<double>("impactdiarrhoea", "diayldnl", parameters.diayldnl);

            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "diamort", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "diamort"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "diayld", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "diayld"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "temp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "temp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdiarrhoea", "bregtmp", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "bregtmp"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactdiarrhoea", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactdiarrhoea", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactdiarrhoea", "regtmp", "climateregional", "regtmp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState));
            ImpactDiarrhoeaState = ss.ConnectToState<Fund.Components.ImpactDiarrhoea.IImpactDiarrhoeaState>(_modelFields, "impactdiarrhoea");
        }

        public void RunImpactDiarrhoeaComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactDiarrhoea.ImpactDiarrhoeaComponent();

            c.Run(clock, ImpactDiarrhoeaState);
        }

        public void InitClimateRegionalState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("climateregional", "bregtmp", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "bregtmp"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("climateregional", "bregstmp", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "bregstmp"));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, double>("climateregional", "scentemp", new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "scentemp"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("climateregional", "inputtemp", "climate", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ClimateRegional.IClimateRegionalState));
            ClimateRegionalState = ss.ConnectToState<Fund.Components.ClimateRegional.IClimateRegionalState>(_modelFields, "climateregional");
        }

        public void RunClimateRegionalComponent(Clock clock)
        {
            var c = new Fund.Components.ClimateRegional.ClimateRegionalComponent();

            c.Run(clock, ClimateRegionalState);
        }

        public void InitImpactTropicalStormsState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impacttropicalstorms", "hurrdamel", parameters.hurrdamel);
            _modelFields.AddNonDimensionalVariable<double>("impacttropicalstorms", "hurrnl", parameters.hurrnl);
            _modelFields.AddNonDimensionalVariable<double>("impacttropicalstorms", "hurrpar", parameters.hurrpar);
            _modelFields.AddNonDimensionalVariable<double>("impacttropicalstorms", "hurrdeadel", parameters.hurrdeadel);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impacttropicalstorms", "hurrbasedam", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "hurrbasedam"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impacttropicalstorms", "hurrbasedead", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "hurrbasedead"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impacttropicalstorms", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impacttropicalstorms", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impacttropicalstorms", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impacttropicalstorms", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impacttropicalstorms", "regstmp", "climateregional", "regstmp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState));
            ImpactTropicalStormsState = ss.ConnectToState<Fund.Components.ImpactTropicalStorms.IImpactTropicalStormsState>(_modelFields, "impacttropicalstorms");
        }

        public void RunImpactTropicalStormsComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactTropicalStorms.ImpactTropicalStormsComponent();

            c.Run(clock, ImpactTropicalStormsState);
        }

        public void InitImpactExtratropicalStormsState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactextratropicalstorms", "co2pre", parameters.co2pre);
            _modelFields.AddNonDimensionalVariable<double>("impactextratropicalstorms", "extratropicalstormsdamel", parameters.extratropicalstormsdamel);
            _modelFields.AddNonDimensionalVariable<double>("impactextratropicalstorms", "extratropicalstormsnl", parameters.extratropicalstormsnl);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "extratropicalstormspar", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "extratropicalstormspar"));
            _modelFields.AddNonDimensionalVariable<double>("impactextratropicalstorms", "extratropicalstormsdeadel", parameters.extratropicalstormsdeadel);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "extratropicalstormsbasedam", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "extratropicalstormsbasedam"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "extratropicalstormsbasedead", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "extratropicalstormsbasedead"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactextratropicalstorms", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactextratropicalstorms", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactextratropicalstorms", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactextratropicalstorms", "acco2", "climate", "acco2");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState));
            ImpactExtratropicalStormsState = ss.ConnectToState<Fund.Components.ImpactExtratropicalStorms.IImpactExtratropicalStormsState>(_modelFields, "impactextratropicalstorms");
        }

        public void RunImpactExtratropicalStormsComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactExtratropicalStorms.ImpactExtratropicalStormsComponent();

            c.Run(clock, ImpactExtratropicalStormsState);
        }

        public void InitImpactSeaLevelRiseState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "incdens", parameters.incdens);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "emcst", parameters.emcst);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "immcst", parameters.immcst);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "dvydl", parameters.dvydl);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "wvel", parameters.wvel);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "wvbm", parameters.wvbm);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "slrwvpopdens0", parameters.slrwvpopdens0);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "wvpdl", parameters.wvpdl);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "wvsl", parameters.wvsl);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "dvbm", parameters.dvbm);
            _modelFields.AddNonDimensionalVariable<double>("impactsealevelrise", "slrwvypc0", parameters.slrwvypc0);

            // Parameters coming from file
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactsealevelrise", "pc", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pc"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactsealevelrise", "slrprtp", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "slrprtp"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactsealevelrise", "wmbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "wmbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactsealevelrise", "dlbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "dlbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactsealevelrise", "wlbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "wlbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactsealevelrise", "coastpd", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "coastpd"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactsealevelrise", "wetmax", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "wetmax"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactsealevelrise", "wetland90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "wetland90"));

            _modelFields.Add2DimensionalParameter<Fund.CommonDimensions.Region, Fund.CommonDimensions.Region, double>("impactsealevelrise", "migrate", new RegionRegionParameterConnector<Fund.CommonDimensions.Region, Fund.CommonDimensions.Region>(parameters, "Migrate"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "sea", "ocean", "sea");
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "ypc", "socioeconomic", "ypc");
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "area", "geography", "area");
            _modelFields.ConnectParameterToVariable("impactsealevelrise", "popdens", "socioeconomic", "popdens");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState));
            ImpactSeaLevelRiseState = ss.ConnectToState<Fund.Components.ImpactSeaLevelRise.IImpactSeaLevelRiseState>(_modelFields, "impactsealevelrise");

        }

        public void RunImpactSeaLevelRiseComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactSeaLevelRise.ImpactSeaLevelRiseComponent();

            c.Run(clock, ImpactSeaLevelRiseState);
        }

        public void InitGeographyState(FundParameter parameters)
        {
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("geography", "area0", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "area0"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("geography", "landloss", "impactsealevelrise", "landloss");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Geography.IGeographyState));
            GeographyState = ss.ConnectToState<Fund.Components.Geography.IGeographyState>(_modelFields, "geography");
        }

        public void RunGeographyComponent(Clock clock)
        {
            var c = new Fund.Components.Geography.GeographyComponent();

            c.Run(clock, GeographyState);
        }

        public void InitEmissionsState(FundParameter parameters)
        {
            _modelFields.AddNonDimensionalVariable<double>("emissions", "so2pop", parameters.so2pop);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "so2inc", parameters.so2inc);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "so2carb", parameters.so2carb);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "sf6gdp", parameters.sf6gdp);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "sf6ypc", parameters.sf6ypc);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "knowpar", parameters.knowpar);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "knowgpar", parameters.knowgpar);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "gwpch4", parameters.gwpch4);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "gwpn2o", parameters.gwpn2o);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "TaxConstant", parameters.TaxConstant);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "TaxEmInt", parameters.TaxEmInt);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "TaxThreshold", parameters.TaxThreshold);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "TaxDepreciation", parameters.TaxDepreciation);
            _modelFields.AddNonDimensionalVariable<double>("emissions", "MaxCostFall", parameters.MaxCostFall);

            _modelFields.ConnectParameterToVariable("emissions", "aeei", "scenariouncertainty", "aeei");
            _modelFields.ConnectParameterToVariable("emissions", "acei", "scenariouncertainty", "acei");
            _modelFields.ConnectParameterToVariable("emissions", "forestemm", "scenariouncertainty", "ForestEmm");

            _modelFields.ConnectParameterToVariable("emissions", "currtax", "globalhotellingtaxation", "currtax");
            _modelFields.ConnectParameterToVariable("emissions", "currtaxch4", "globalhotellingtaxation", "currtaxch4");
            _modelFields.ConnectParameterToVariable("emissions", "currtaxn2o", "globalhotellingtaxation", "currtaxn2o");

            _modelFields.ConnectParameterToVariable("emissions", "pgrowth", "scenariouncertainty", "pgrowth");
            _modelFields.ConnectParameterToVariable("emissions", "ypcgrowth", "scenariouncertainty", "ypcgrowth");
            _modelFields.ConnectParameterToVariable("emissions", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("emissions", "population", "socioeconomic", "population");


            // Parameters coming from file
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "taxmp", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "taxmp"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "gdp0", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp0"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "sf60", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "sf60"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "GDP90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, double>("emissions", "ch4em", new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ch4em"));
            _modelFields.Add2DimensionalParameter<Timestep, Fund.CommonDimensions.Region, double>("emissions", "n2oem", new YearRegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "n2oem"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "ch4par1", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ch4par1"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "ch4par2", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ch4par2"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "n2opar1", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "n2opar1"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("emissions", "n2opar2", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "n2opar2"));

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.Emissions.IEmissionsState));
            EmissionsState = ss.ConnectToState<Fund.Components.Emissions.IEmissionsState>(_modelFields, "emissions");

            // Initial values, should eventually also come from parameter file
            var t0 = Timestep.FromSimulationYear(0);
            foreach (Fund.CommonDimensions.Region r in Enum.GetValues(typeof(Fund.CommonDimensions.Region)))
            {
                EmissionsState.energint[t0, r] = 1;
                EmissionsState.energuse[t0, r] = _parameters.gdp0[(int)r];
                EmissionsState.emissint[t0, r] = _parameters.emissint0[(int)r];
                EmissionsState.so2[t0, r] = _parameters.so20[(int)r];
                EmissionsState.emission[t0, r] = EmissionsState.emissint[t0, r] / EmissionsState.energuse[t0, r];
                EmissionsState.ch4cost[t0, r] = 0;
                EmissionsState.n2ocost[t0, r] = 0;
                EmissionsState.ryg[t0, r] = 0;
                EmissionsState.reei[t0, r] = 0;
                EmissionsState.rcei[t0, r] = 0;
                EmissionsState.seei[t0, r] = 0;
                EmissionsState.scei[t0, r] = 0;
                EmissionsState.co2red[t0, r] = 0;
                EmissionsState.know[t0, r] = 1;
                EmissionsState.ch4red[t0, r] = 0;
                EmissionsState.n2ored[t0, r] = 0;
            }
            EmissionsState.globknow[t0] = 1;

            //SocioEconomicState.minint[t0]=Double.PositiveInfinity;
            var minint = double.PositiveInfinity;
            foreach (Fund.CommonDimensions.Region r in Enum.GetValues(typeof(Fund.CommonDimensions.Region)))
            {
                if (EmissionsState.emission[t0, r] / EmissionsState.income[t0, r] < minint)
                    minint = EmissionsState.emission[t0, r] / EmissionsState.income[t0, r];
            }
            EmissionsState.minint[t0] = minint;

        }

        public void RunEmissionsComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var c = new Fund.Components.Emissions.EmissionsComponent();

            // Run SocioEconomic component
            c.Run(clock, EmissionsState);
        }

        public void InitTerrestrialBiosphereState(FundParameter parameters)
        {
            _modelFields.AddNonDimensionalVariable<double>("terrestrialbiosphere", "TerrCO2Sens", parameters.TerrCO2Sens);
            _modelFields.AddNonDimensionalVariable<double>("terrestrialbiosphere", "TerrCO2Stock0", parameters.TerrCO2Stock0);

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("terrestrialbiosphere", "temp", "climate", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState));
            TerrestrialBiosphereState = ss.ConnectToState<Fund.Components.TerrestrialBiosphere.ITerrestrialBiosphereState>(_modelFields, "terrestrialbiosphere");
        }

        public void RunTerrestrialBiosphereComponent(Clock clock)
        {
            // Set up SocioEconomic component
            var c = new Fund.Components.TerrestrialBiosphere.TerrestrialBiosphereComponent();

            // Run SocioEconomic component
            c.Run(clock, TerrestrialBiosphereState);
        }

        public void InitImpactForestsState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactforests", "forel", parameters.forel);
            _modelFields.AddNonDimensionalVariable<double>("impactforests", "fornl", parameters.fornl);
            _modelFields.AddNonDimensionalVariable<double>("impactforests", "co2pre", parameters.co2pre);
            _modelFields.AddNonDimensionalVariable<double>("impactforests", "forco2", parameters.forco2);
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactforests", "forbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "forbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactforests", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactforests", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactforests", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactforests", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactforests", "temp", "climateregional", "temp");
            _modelFields.ConnectParameterToVariable("impactforests", "acco2", "climate", "acco2");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactForests.IImpactForestsState));
            ImpactForestsState = ss.ConnectToState<Fund.Components.ImpactForests.IImpactForestsState>(_modelFields, "impactforests");
        }

        public void RunImpactForestsComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactForests.ImpactForests();

            c.Run(clock, ImpactForestsState);
        }

        public void InitImpactVectorBorneDiseasesState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactvectorbornediseases", "dfnl", parameters.dfnl);
            _modelFields.AddNonDimensionalVariable<double>("impactvectorbornediseases", "vbel", parameters.vbel);
            _modelFields.AddNonDimensionalVariable<double>("impactvectorbornediseases", "smnl", parameters.smnl);
            _modelFields.AddNonDimensionalVariable<double>("impactvectorbornediseases", "malnl", parameters.malnl);

            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "dfbs", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "dfbs"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "dfch", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "dfch"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "smbs", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "smbs"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "smch", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "smch"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "malbs", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "malbs"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "malch", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "malch"));

            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "gdp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "gdp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactvectorbornediseases", "pop90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "pop90"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactvectorbornediseases", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactvectorbornediseases", "income", "socioeconomic", "income");
            _modelFields.ConnectParameterToVariable("impactvectorbornediseases", "temp", "climateregional", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState));
            ImpactVectorBorneDiseases = ss.ConnectToState<Fund.Components.ImpactVectorBorneDiseases.IImpactVectorBorneDiseasesState>(_modelFields, "impactvectorbornediseases");
        }

        public void RunImpactVectorBorneDiseasesComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactVectorBorneDiseases.ImpactVectorBorneDiseasesComponent();

            c.Run(clock, ImpactVectorBorneDiseases);
        }

        public void InitImpactCardiovascularRespiratoryState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactcardiovascularrespiratory", "cvlin", parameters.cvlin);
            _modelFields.AddNonDimensionalVariable<double>("impactcardiovascularrespiratory", "rlin", parameters.rlin);
            _modelFields.AddNonDimensionalVariable<double>("impactcardiovascularrespiratory", "maxcardvasc", parameters.maxcardvasc);

            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "cardvasc90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "cardvasc90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "plus90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "plus90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "resp90", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "resp90"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "chplbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "chplbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "chmlbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "chmlbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "chpqbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "chpqbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "chmqbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "chmqbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "rlbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "rlbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "rqbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "rqbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "ccplbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ccplbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "ccmlbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ccmlbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "ccpqbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ccpqbm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactcardiovascularrespiratory", "ccmqbm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "ccmqbm"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactcardiovascularrespiratory", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactcardiovascularrespiratory", "plus", "socioeconomic", "plus");
            _modelFields.ConnectParameterToVariable("impactcardiovascularrespiratory", "urbpop", "socioeconomic", "urbpop");
            _modelFields.ConnectParameterToVariable("impactcardiovascularrespiratory", "temp", "climateregional", "temp");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState));
            ImpactCardiovascularRespiratoryState = ss.ConnectToState<Fund.Components.ImpactCardiovascularRespiratory.IImpactCardiovascularRespiratoryState>(_modelFields, "impactcardiovascularrespiratory");
        }

        public void RunImpactCardiovascularRespiratoryComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactCardiovascularRespiratory.ImpactCardiovascularRespiratoryComponent();

            c.Run(clock, ImpactCardiovascularRespiratoryState);
        }

        public void InitImpactDeathMorbidityState(FundParameter parameters)
        {
            // Constant parameters
            _modelFields.AddNonDimensionalVariable<double>("impactdeathmorbidity", "vslbm", parameters.vslbm);
            _modelFields.AddNonDimensionalVariable<double>("impactdeathmorbidity", "vslel", parameters.vslel);
            _modelFields.AddNonDimensionalVariable<double>("impactdeathmorbidity", "vmorbbm", parameters.vmorbbm);
            _modelFields.AddNonDimensionalVariable<double>("impactdeathmorbidity", "vmorbel", parameters.vmorbel);
            _modelFields.AddNonDimensionalVariable<double>("impactdeathmorbidity", "vslypc0", parameters.vslypc0);
            _modelFields.AddNonDimensionalVariable<double>("impactdeathmorbidity", "vmorbypc0", parameters.vmorbypc0);


            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2ld", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2ld"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2ls", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2ls"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2lm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2lm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2lc", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2lc"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2lr", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2lr"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2dd", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2dd"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2ds", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2ds"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2dm", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2dm"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2dc", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2dc"));
            _modelFields.Add1DimensionalParameter<Fund.CommonDimensions.Region, double>("impactdeathmorbidity", "d2dr", new RegionParameterConnector<Fund.CommonDimensions.Region>(parameters, "d2dr"));

            // Parameters coming from different components, directly coupled
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "population", "socioeconomic", "population");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "ypc", "socioeconomic", "ypc");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "dengue", "impactvectorbornediseases", "dengue");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "schisto", "impactvectorbornediseases", "schisto");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "malaria", "impactvectorbornediseases", "malaria");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "cardheat", "impactcardiovascularrespiratory", "cardheat");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "cardcold", "impactcardiovascularrespiratory", "cardcold");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "resp", "impactcardiovascularrespiratory", "resp");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "diadead", "impactdiarrhoea", "diadead");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "hurrdead", "impacttropicalstorms", "hurrdead");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "extratropicalstormsdead", "impactextratropicalstorms", "extratropicalstormsdead");
            _modelFields.ConnectParameterToVariable("impactdeathmorbidity", "diasick", "impactdiarrhoea", "diasick");

            // Create proxy
            var ss = Esmf.ComponentStructure.StateStructure.LoadFromInterface(typeof(Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState));
            ImpactDeathMorbidityState = ss.ConnectToState<Fund.Components.ImpactDeathMorbidity.IImpactDeathMorbidityState>(_modelFields, "impactdeathmorbidity");
        }

        public void RunImpactDeathMorbidityComponent(Clock clock)
        {
            var c = new Fund.Components.ImpactDeathMorbidity.ImpactDeathMorbidityComponent();

            c.Run(clock, ImpactDeathMorbidityState);
        }

    }
}