﻿// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.5
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Esmf;
using Fund.CommonDimensions;

namespace Fund.Components.Climate
{

    /// <summary>State for the climate component</summary>
    public interface IClimateState
    {
        /// <summary>co2 emissions</summary>
        IParameter1Dimensional<Timestep, double> mco2 { get; }

        IVariable1Dimensional<Timestep, double> globc { get; }

        /// <summary>Carbon box 1</summary>
        IVariable1Dimensional<Timestep, double> cbox1 { get; }

        /// <summary>Carbon box 2</summary>
        IVariable1Dimensional<Timestep, double> cbox2 { get; }

        /// <summary>Carbon box 3</summary>
        IVariable1Dimensional<Timestep, double> cbox3 { get; }

        /// <summary>Carbon box 4</summary>
        IVariable1Dimensional<Timestep, double> cbox4 { get; }

        /// <summary>Carbon box 5</summary>
        IVariable1Dimensional<Timestep, double> cbox5 { get; }

        /// <summary>Carbon decay in box 1</summary>
        double co2decay1 { get; set; }

        /// <summary>Carbon decay in box 2</summary>
        double co2decay2 { get; set; }

        /// <summary>Carbon decay in box 3</summary>
        double co2decay3 { get; set; }

        /// <summary>Carbon decay in box 4</summary>
        double co2decay4 { get; set; }

        /// <summary>Carbon decay in box 5</summary>
        double co2decay5 { get; set; }

        /// <summary>Carbon decay in box 1</summary>
        double lifeco1 { get; }

        /// <summary>Carbon decay in box 2</summary>
        double lifeco2 { get; }

        /// <summary>Carbon decay in box 3</summary>
        double lifeco3 { get; }

        /// <summary>Carbon decay in box 4</summary>
        double lifeco4 { get; }

        /// <summary>Carbon decay in box 5</summary>
        double lifeco5 { get; }

        /// <summary>Fraction of carbon emission in box 1</summary>  
        double co2frac1 { get; }

        /// <summary>Fraction of carbon emission in box 2</summary>  
        double co2frac2 { get; }

        /// <summary>Fraction of carbon emission in box 3</summary>  
        double co2frac3 { get; }

        /// <summary>Fraction of carbon emission in box 4</summary>  
        double co2frac4 { get; }

        /// <summary>Fraction of carbon emission in box 5</summary>  
        double co2frac5 { get; }

        /// <summary>CO2 emissions from the terrestrial biosphere </summary>
        IParameter1Dimensional<Timestep, double> TerrestrialCO2 { get; }

        /// <summary>Atmospheric CO2 concentration</summary>
        IVariable1Dimensional<Timestep, double> acco2 { get; }

        /// <summary> CO2 pre industrial</summary>
        double co2pre { get; }

        /// <summary>Global CH4 emissions</summary>
        IParameter1Dimensional<Timestep, double> globch4 { get; }

        /// <summary>Atmospheric CH4 concentration</summary>
        IVariable1Dimensional<Timestep, double> acch4 { get; }

        /// <summary>CH4 decay</summary>
        double ch4decay { get; set; }

        /// <summary></summary>
        double lifech4 { get; }

        /// <summary> CH4 pre industrial</summary>
        double ch4pre { get; }

        /// <summary>Global N2O emissions</summary>
        IParameter1Dimensional<Timestep, double> globn2o { get; }

        /// <summary>Atmospheric N2O concentration</summary>
        IVariable1Dimensional<Timestep, double> acn2o { get; }

        /// <summary>N2O decay</summary>
        double n2odecay { get; set; }

        /// <summary></summary>
        Double lifen2o { get; }

        /// <summary>N2o pre industrial</summary>
        double n2opre { get; }

        /// <summary> Global SF6 emissions</summary>
        IParameter1Dimensional<Timestep, double> globsf6 { get; }

        /// <summary>Atmospheric SF6 concentrations</summary>
        IVariable1Dimensional<Timestep, double> acsf6 { get; }

        /// <summary>SF6 pre industrial</summary>
        double sf6pre { get; }

        /// <summary>SF6 decay</summary>
        double sf6decay { get; set; }

        /// <summary></summary>
        double lifesf6 { get; }

        /// <summary>Global SO2 emissions</summary>
        IParameter1Dimensional<Timestep, double> globso2 { get; }

        /// <summary>Atmospheric SO2 concentration</summary>
        IVariable1Dimensional<Timestep, double> acso2 { get; }

        /// <summary>Direct radiative forcing by sulphate aerosols</summary>
        Double so2dir { get; }

        /// <summary>Indirect radiative forcing by sulphate aerosols</summary>
        double so2ind { get; }

        /// <summary>Radiative forcing from CO2</summary>
        IVariable1Dimensional<Timestep, double> rfCO2 { get; }

        /// <summary>Radiative forcing from CH4</summary>
        IVariable1Dimensional<Timestep, double> rfCH4 { get; }

        /// <summary>Radiative forcing from N2O</summary>
        IVariable1Dimensional<Timestep, double> rfN2O { get; }

        /// <summary>Radiative forcing from N2O</summary>
        IVariable1Dimensional<Timestep, double> rfSF6 { get; }

        /// <summary>Radiative forcing from SO2</summary>
        IVariable1Dimensional<Timestep, double> rfSO2 { get; }

        /// <summary>Total radiative forcing</summary>
        IVariable1Dimensional<Timestep, double> radforc { get; }

        /// <summary>Total radiative forcing according to EMF22 definition</summary>
        IVariable1Dimensional<Timestep, double> rfEMF22 { get; }

        /// <summary> Maximal radiative forcing</summary>
        Double maxrf { get; set; }

        /// <summary>Average global temperature</summary>
        IVariable1Dimensional<Timestep, double> temp { get; }

        /// <summary>LifeTempConst</summary>
        double LifeTempConst { get; }

        /// <summary>LifeTempLin</summary>
        double LifeTempLin { get; }

        /// <summary>Climate sensitivity</summary>
        double ClimateSensitivity { get; }

        bool addmp { get; }
        IParameter1Dimensional<Timestep, double> addco2 { get; }
        IParameter1Dimensional<Timestep, double> addch4 { get; }
        IParameter1Dimensional<Timestep, double> addn2o { get; }
        IParameter1Dimensional<Timestep, double> addsf6 { get; }

        double ch4add { get; }
        double n2oadd { get; }
        double sf6add { get; }
        IParameter1Dimensional<Timestep, double> RunAwayCH4 { get; }
    }
    public class ClimateComponent
    {

        private double Interact(double M, double N)
        {
            double hulp;

            hulp = 1.0 + Math.Pow(M * N, 0.75) * 2.01E-5 + Math.Pow(M * N, 1.52) * M * 5.31E-15;
            return 0.47 * Math.Log(hulp);
        }

        public void Run(Clock clock, IClimateState state)
        {
            // create shortcuts for commonly accessed data
            var s = state;
            var t = clock.Current;

            s.co2decay1 = s.lifeco1;
            s.co2decay2 = Math.Exp(-1.0 / s.lifeco2);
            s.co2decay3 = Math.Exp(-1.0 / s.lifeco3);
            s.co2decay4 = Math.Exp(-1.0 / s.lifeco4);
            s.co2decay5 = Math.Exp(-1.0 / s.lifeco5);

            s.ch4decay = 1.0 / s.lifech4;
            s.n2odecay = 1.0 / s.lifen2o;
            s.sf6decay = 1.0 / s.lifesf6;

            // 
            double CH4N2O = Interact(s.ch4pre, s.n2opre);

            s.globc[t] = s.mco2[t] + s.TerrestrialCO2[t];

            // Calculate CO2 concentrations
            s.cbox1[t] = s.cbox1[t - 1] * s.co2decay1 + 0.000471 * s.co2frac1 * (s.mco2[t] + s.TerrestrialCO2[t] + (s.addmp ? s.addco2[t] : 0.0));
            s.cbox2[t] = s.cbox2[t - 1] * s.co2decay2 + 0.000471 * s.co2frac2 * (s.mco2[t] + s.TerrestrialCO2[t] + (s.addmp ? s.addco2[t] : 0.0));
            s.cbox3[t] = s.cbox3[t - 1] * s.co2decay3 + 0.000471 * s.co2frac3 * (s.mco2[t] + s.TerrestrialCO2[t] + (s.addmp ? s.addco2[t] : 0.0));
            s.cbox4[t] = s.cbox4[t - 1] * s.co2decay4 + 0.000471 * s.co2frac4 * (s.mco2[t] + s.TerrestrialCO2[t] + (s.addmp ? s.addco2[t] : 0.0));
            s.cbox5[t] = s.cbox5[t - 1] * s.co2decay5 + 0.000471 * s.co2frac5 * (s.mco2[t] + s.TerrestrialCO2[t] + (s.addmp ? s.addco2[t] : 0.0));

            s.acco2[t] = s.cbox1[t] + s.cbox2[t] + s.cbox3[t] + s.cbox4[t] + s.cbox5[t];

            // Calculate CH4 concentrations
            var globch4 = s.globch4[t] + (s.addmp ? s.addch4[t] : 0.0) + s.RunAwayCH4[t] + (t.Value > 50 ? s.ch4add * (t.Value - 50) : 0.0);
            if (globch4 < 0)
                globch4 = 0;
            s.acch4[t] = s.acch4[t - 1] + 0.3597 * globch4 - s.ch4decay * (s.acch4[t - 1] - s.ch4pre);

            if (s.acch4[t] < 0)
                throw new ApplicationException("ch4 atmospheric concentration out of range");

            // Calculate N2O concentrations
            var globn2o = s.globn2o[t] + (s.addmp ? s.addn2o[t] : 0.0) + (t.Value > 50 ? s.n2oadd * (t.Value - 50) : 0);
            s.acn2o[t] = s.acn2o[t - 1] + 0.2079 * globn2o - s.n2odecay * (s.acn2o[t - 1] - s.n2opre);

            if (s.acn2o[t] < 0)
                throw new ApplicationException("n2o atmospheric concentration out of range");

            // Calculate SF6 concentrations   
            var globsf6 = s.globsf6[t] + (s.addmp ? s.addsf6[t] : 0.0) + (t.Value > 50 ? s.sf6add * (t.Value - 50) : 0.0);
            if (globsf6 < 0)
                globsf6 = 0;
            s.acsf6[t] = s.sf6pre + (s.acsf6[t - 1] - s.sf6pre) * (1 - s.sf6decay) + globsf6 / 25.1;

            if (s.acsf6[t] < 0)
                throw new ApplicationException("sf6 atmospheric concentration out of range");

            // Calculate SO2 concentrations
            s.acso2[t] = s.globso2[t];

            // Calculate radiative forcing        

            s.rfCO2[t] = 5.35 * Math.Log(s.acco2[t] / s.co2pre);

            s.rfCH4[t] = 0.036 * (Math.Sqrt(s.acch4[t]) - Math.Sqrt(s.ch4pre))
              - Interact(s.acch4[t], s.n2opre) + CH4N2O;

            s.rfN2O[t] = 0.14 * (Math.Sqrt(s.acn2o[t]) - Math.Sqrt(s.n2opre))
              - Interact(s.ch4pre, s.acn2o[t]) + CH4N2O;

            s.rfSF6[t] = 0.00052 * (s.acsf6[t] - s.sf6pre);

            s.rfSO2[t] = s.so2dir * s.acso2[t] / 14.6 + s.so2ind * Math.Log(1 + s.acso2[t] / 34.4) / Math.Log(1 + 14.6 / 34.4) - 0.9;
            // s.rfSO2[t]=s.so2dir*s.globso2[t]/14.6 + s.so2ind*ln(1+s.globso2[t]/34.4)/ln(1+14.6/34.4) - 0.9;

            s.radforc[t] = s.rfCO2[t] + s.rfCH4[t] + s.rfN2O[t] + s.rfSF6[t] - s.rfSO2[t];

            s.rfEMF22[t] = s.rfCO2[t] + s.rfCH4[t] + s.rfN2O[t];

            if (s.radforc[t] > s.maxrf)
                s.maxrf = s.radforc[t];

            var LifeTemp = Math.Max(s.LifeTempConst + s.LifeTempLin * s.ClimateSensitivity, 1.0);

            var delaytemp = 1.0 / LifeTemp;

            var temps = s.ClimateSensitivity / 5.35 / Math.Log(2.0);

            // Calculate temperature          
            var dtemp = delaytemp * temps * s.radforc[t] - delaytemp * s.temp[t - 1];

            s.temp[t] = s.temp[t - 1] + dtemp;
        }
    }
}
